@@ -71,6 +71,12 @@ void KNotesMigrator::notesResourceCreated(KJob * job)
   QString resourcePath = kresCfg.readEntry( "NotesURL" );
   KUrl url(resourcePath);
 
+  if (!QFile::exists(url.toLocalFile()))
+  {
+    migrationCompleted(m_agentInstance);
+    return;
+  }
+
   m_notesResource = new KCal::CalendarLocal(QString());
 
   bool success = m_notesResource->load(url.toLocalFile());
@@ -105,6 +111,7 @@ void KNotesMigrator::notesResourceCreated(KJob * job)
 
 void KNotesMigrator::syncDone(KJob *job)
 {
+  Q_UNUSED( job );
   emit message( Info, i18n( "Instance \"%1\" synchronized" , m_agentInstance.identifier() ) );
 
   CollectionFetchJob *collectionFetchJob = new CollectionFetchJob( Collection::root(), CollectionFetchJob::FirstLevel, this );
@@ -166,6 +173,7 @@ void KNotesMigrator::startMigration()
 
 void KNotesMigrator::newResourceFilled(KJob* job)
 {
+  Q_UNUSED( job );
   migrationCompleted( m_agentInstance );
 }
 
