@@ -131,6 +131,11 @@ void AddItemTask::triggerSearchJob( KIMAP::Session *session )
     search->addSearchCriteria( KIMAP::SearchJob::New );
 
     UidNextAttribute *uidNext = collection().attribute<UidNextAttribute>();
+    if ( !uidNext ){
+      cancelTask( i18n("Could not determine the UID for the newly created message on the server") );
+      search->deleteLater();
+      return;
+    }
     KIMAP::ImapInterval interval( uidNext->uidNext() );
 
     search->addSearchCriteria( KIMAP::SearchJob::Uid, interval.toImapSequence() );
